; updates screen
; takes:none
; destroys:de,hl,bc
Update:
	ld hl,backBuffer1
	ld de,frontBuffer1
	ld bc,768
	ldir
	ld hl,backBuffer2
	ld de,frontBuffer2
	ld bc,768
	ldir
	ret

RevUpdate:
	ld hl,frontBuffer1
	ld de,backBuffer1
	ld bc,768
	ldir
	ld hl,frontBuffer2
	ld de,backBuffer2
	ld bc,768
	ldir
	ret


; draws a gray masked sprite to the front buffers
; hl - data      mask - b1 - b2 - mask - b1 - b2.....
; bc - positions
; ix - buffer1
; iy - buffer2
graySpriteMasked:
	call _getBCinIXIY
	ld a,b
	and 7
	jr z,_graySpriteMasked_Aligned
	ld c,a
	ld b,8
	_graySpriteMasked_Loop:
		push bc
			ld b,c
			
			ld d,(hl)
			ld e,0
			inc hl
			
			ld c,(hl)
			inc hl
			
			ld a,(hl)
			inc hl
			
			push hl
				ld h,a
				ld l,$ff
				ld a,$ff
				_graySpriteMasked_Shift:  ; shifting: de mask, ca sprite data 1, hl sprite data 2
					srl d
					rr e
					
					scf
					rr c
					rra
					
					rr h
					rr l
					djnz _graySpriteMasked_Shift
				
				ld b,a
				
				ld a,(ix+1)
				or e
				and b
				ld (ix+1),a
				
				ld a,(iy+1)
				or e
				and l
				ld (iy+1),a
				
				
				ld a,(ix)
				or d
				and c
				ld (ix),a
				
				ld a,(iy)
				or d
				and h
				ld (iy),a
				
				ld de,12
				add ix,de
				add iy,de
			pop hl
		pop bc
		djnz _graySpriteMasked_Loop
	ret
_graySpriteMasked_Aligned:
	ld b,8
	ld de,12
	@:
		ld c,(hl)
		inc hl
		
		ld a,(ix)
		or c
		and (hl)
		ld (ix),a
		inc hl
		
		ld a,(iy)
		or c
		and (hl)
		ld (iy),a
		inc hl
		
		add ix,de
		add iy,de
		
		djnz {-1@}
	ret


GetSprite:
; bc - positions
; hl - 16 bytes to copy it to
; ix - buffer1
; iy - buffer2
	call _getBCinIXIY
	
	ld a,b
	and	7
	ld	b,8
	jr	nz,___GetSpriteUnaligned
	
	ld	de,12
	___GetSpriteAligned:
		ld a,(ix)
		ld (hl),a
		inc hl
		
		ld a,(iy)
		ld (hl),a
		inc hl
		
		add ix,de
		add iy,de
		djnz	___GetSpriteAligned
	ret
	
	___GetSpriteUnaligned:
		ex de,hl
			ld h,(ix)
			ld l,(ix+1)
			ld c,a
			___GetSpriteShift:
				add hl,hl
				dec c
				jr nz,___GetSpriteShift
		ex de,hl
		ld (hl),d
		inc hl
		
		ex de,hl
			ld h,(iy)
			ld l,(iy+1)
			ld c,a
			___GetSpriteShift2:
				add hl,hl
				dec c
				jr nz,___GetSpriteShift2
		ex de,hl
		ld (hl),d
		inc hl
		
		ld de,12
		add ix,de
		add iy,de
		djnz ___GetSpriteUnaligned
	ret

_getBCinIXIY:
; gets the pixel-bc coordinates as offsets added to ix and iy
; preserves hl
	push hl
		ld d,0
		ld e,c
		ld h,d
		ld l,e
		add hl,hl
		add hl,de
		add hl,hl
		add hl,hl
		ld e,b
		srl e
		srl e
		srl e
		add hl,de
		ex de,hl
		add ix,de
		add iy,de
	pop hl
	ret



drawPxlOR:
; bc - positions (pixel-wise)
; ix - buffer1
; iy - buffer2
; e - ignore buffer 2?
	push de
		push bc
			ld d,0
			ld e,c
			ld h,d
			ld l,e
			add hl,hl
			add hl,de
			add hl,hl
			add hl,hl
			ld e,b
			srl e
			srl e
			srl e
			add hl,de
			ex de,hl
			add ix,de
			add iy,de
		pop bc
		ld a,b
		and 7
		jr z,_drawPxlOR_aligned
		ld b,a
		ld a,%10000000
		@:
			srl a
			djnz {-1@}
		ld b,a
	_drawPxlOR_aligned_back:
		ld a,(ix)
		or b
		ld (ix),a
	pop de
	xor a
	cp e
	ret nz
	ld a,(iy)
	or b
	ld (iy),a
	ret
_drawPxlOR_aligned:
	ld b,%10000000
	jr _drawPxlOR_aligned_back
