GrayTuner:
	bcall(_ClrAllBuffers)
	
	
	ld hl,_GrayTuner_Row1
	ld de,backBuffer1 + 6*12
	ld bc,12
	ldir
	
	ld hl,_GrayTuner_Row2
	ld de,backBuffer2 + 6*12
	ld bc,12
	ldir
	
	ld hl,backBuffer1 + 6*12
	ld de,backBuffer1 + 7*12
	ld bc,768 - (7*12)
	ldir
	
	ld hl,backBuffer2 + 6*12
	ld de,backBuffer2 + 7*12
	ld bc,768 - (7*12)
	ldir
	
	bcall(_Update)
	ld bc,0
	ld a,1
	ld hl,string_graytuner
	bcall(_Text)
	
	ld hl,(GSCounter)
	ld (tmp1),hl
_GrayTuner_Keyloop:
	bcall(_Wait_Key_Release)
	bcall(_Wait_Key_Input)
	
	bit 0,a ; down key, decrease contrast
	jr nz,{+1@}
	push af
		ld a,(GSContrast)
		dec a
		and %00111111
		ld (GSContrast),a
		or %11000000
		out ($10),a
	pop af
	@:
	bit 3,a ; up key, increase contrast
	jr nz,{+1@}
	push af
		ld a,(GSContrast)
		inc a
		and %00111111
		ld (GSContrast),a
		or %11000000
		out ($10),a
	pop af
	@:
	
	bit 2,a ; right key, decrease delay
	jr nz,{+1@}
	push af
		ld hl,GSCounter
		dec (hl)
		
		jr nz,$+3
		inc (hl)
		
		call DisableInterrupts
		call RestoreInterrupts
	pop af
	@:
	bit 1,a ; left key, increase delay
	jr nz,{+1@}
	push af
		ld hl,GSCounter
		inc (hl)
		
		jr nz,$+3
		dec (hl)
		
		call DisableInterrupts
		call RestoreInterrupts
	pop af
	@:
	
	bit 7,a
	jr z,_GrayTuner_Exit_nosave
	
	cpl
	and %01100000
	jr z,_GrayTuner_Keyloop
	; time to save it!
	call DisableInterrupts
	
	ld hl,GSAppvarName ; we know that the appvar is in flash, so let's unarchie it!
	rst rMov9ToOP1
	bcall(_Arc_Unarc)
	inc de ; header
	inc de
	ld hl,GSCounter
	ld bc,2
	ldir
	
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_Arc_Unarc)
	call RestoreScreenSettings

	call RestoreInterrupts
	ld a,%10111110 ; listen to key groups
	out (1),a
	bcall(_Wait_Key_Release)
	ret
_GrayTuner_Exit_nosave:
	ld hl,(tmp1)
	ld (GSCounter),hl
	
	call DisableInterrupts
	
	ld a,(GSContrast)
	or %11000000
	out ($10),a
	
	call RestoreInterrupts
	
	
	bcall(_Wait_Key_Release)
	ret
_GrayTuner_Row1:
	.db $00,$00,$00,$00,$00,$00,$FF,$FF,$FF,$FF,$FF,$FF
_GrayTuner_Row2:
	.db $00,$00,$00,$FF,$FF,$FF,$00,$00,$00,$FF,$FF,$FF

LoadInterrupt:
.relocate keyToStr
	push af
	in a,(6)
	push af
LoadInterrupt_page = $+1
	  ld a,00
	  out (6),a
	  call Interrupt
	pop af
	out (6),a
	pop af
	ei
	ret
.endrelocate
LoadInterruptSize = $-LoadInterrupt

.if LoadInterruptSize > 18
.fail "Interrupt handler is too large! (18 bytes max)"
.endif


GSAppvarName:
	.db AppVarObj,"GS",0
GSAppvarNameLen = $ - GSAppvarName - 2

loadInterruptSettings:
	

	call DisableInterrupts
	
	xor a ; we'll keep track here if we need to prompt with the settings
	ld (tmp1),a
	
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_ChkFindSym)
	jr c,_loadInterruptSettings_No_AppVar
	
	ld a,b
	or a
	jr nz,_loadInterruptSettings_AppVar_in_Flash
	jr _loadInterruptSettings_AppVar_in_RAM
_loadInterruptSettings_No_AppVar:
	ld a,1 ; ok, we need to display the settings
	ld (tmp1),a
	ld hl,GSAppvarName ; create the appvar
	rst rMov9ToOP1
	ld hl,23
	bcall(_CreateAppVar)
	inc de ; header
	inc de
	ex de,hl
	
	in a,(2)
	bit 7,a
	jr z,{+1@}
		ld (hl),184
		jr {+2@}
	@:
		ld (hl),4
	@:
	inc hl
	
	ld a,(contrast) ; fetch os contrast as default
	add a,24
	ld (hl),a
	
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_ChkFindSym)
	
_loadInterruptSettings_AppVar_in_RAM:
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_Arc_Unarc)
	bcall(_ChkFindSym)
_loadInterruptSettings_AppVar_in_Flash:

	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_ChkFindSym)
	ex de,hl
	
	ld a,1+2+7+GSAppvarNameLen+2
	
	@:
		bcall(_LoadDEIndPaged)
		dec a
		jr nz,{-1@}
	ld a,b
	
	ld bc,2
	ld de,GSCounter
	bcall(_FlashToRAM)
	ld a,(GSContrast)
	or %11000000
	out ($10),a
	call RestoreInterrupts
	
	ld a,%10111110 ; listen to key groups
	out (1),a
	
	ld a,(tmp1)
	or a
	ret z
	jp GrayTuner


	
DisableInterrupts:

	xor a
	out ($30),a
	
	di
	
	im 1
	ret

RestoreInterrupts:
	ld hl, $8000
	ld de, $8001
	ld (hl), $85
	ld bc, 256
	ldir
	
	in a,(2)
	bit 7,a
	jr z,{+1@}
	
	
		ld a,$40
		out ($30),a
		
		ld a,3
		out ($31),a
		
		ld a,(GSCounter)
		out ($32),a
		
		ld a,1
		out ($20),a
		jr {+2@}
	@:
		ld a,(GSCounter)
		ld (interruptCounter),a
		xor a
		out (4),a
	@:
	im 2
	ei
	ret


Interrupt:
	exx
	xor a
	out (3),a ; ack interrupts
	
	in a,(2)
	bit 7,a
	jr nz,{+2@}
		ld hl,interruptCounter
		dec (hl)
		jr z,{+1@}
			ld a,(interruptMask)
			out (3),a
			exx
			ret
		@:
		ld a,(GSCounter)
		ld (interruptCounter),a
		jr {+2@}
	@:
		xor a
		out ($20),a
	@:
	call Disp4Lvl
	
	ld hl,interruptMask
	set 7,(hl) ; set it so that Halt can check for it
	ld a,(hl)
	out (3),a ; reenable interrupts
	
	bit 1,a
	jr nz,{+1@}
		ld a,3
		out ($31),a
		ld a,1
		out ($20),a
	@:
	exx
	
	ret
Disp4Lvl:
	ld    a,%11011011
	or    a
	ld    hl,flags+asm_flag2
	inc   (hl)
	jr    z,__Disp4Lvlskip
	rra
	ld    b,(hl)
	inc   b
	jr    z,__Disp4Lvlskip
	rra
	ld    (hl),-2
__Disp4LvlSkip:
	ex    af,af'
	
	
	ld    hl,frontBuffer2
	ld    de,frontBuffer1
	ld    a,$80
__Disp4LvlEntry:
	out   ($10),a		     ;76+cc into, 71cc loop
	push  af
		ex    (sp),hl		     ;waste
		ex    (sp),hl		     ;waste
		nop			   ;waste
		ld    a,$20
		out   ($10),a		     ;71cc into
		ld    b,12
		__Disp4LvlLoop:
			ex    af,af'
			rra
			ld    c,a
			ex    af,af'
			ld    a,(de)
			xor   (hl)
			and   c
			xor   (hl)
			inc   de
			inc   hl
			out   ($11),a		     ;71cc into, 77cc loop
			djnz  __Disp4LvlLoop
		inc   bc		    ;waste
		ex    af,af'
		rra
		ex    af,af'
	pop   af
	inc   a
	bit   6,a
	jr    z,__Disp4LvlEntry
__Disp4LvlDone:
	
	
	ret
