Reuben_add_HP:
; takes: hl
; destroys: hl, de
; preserves: bc, ix, iy, a
	ex de,hl
	ld hl,(reuben_hp)
	
	add hl,de
	ex de,hl
	ld hl,(reuben_maxhp)
	or a
	sbc hl,de
	jr nc,{+}   ; carry means just load the max hp into current hp
		ld hl,(reuben_maxhp)
		ld (reuben_hp),hl
		ret
	+:
	
	ex de,hl
	ld (reuben_hp),hl
	ret

Reuben_add_Gold:
; takes: hl
; destroys: hl, de
; preserves: bc, ix, iy, a
	ex de,hl
	ld hl,(reuben_gold)
	
	add hl,de
	ex de,hl
	ld hl,(reuben_maxgold)
	or a
	sbc hl,de
	jr nc,{+}
		ld hl,(reuben_maxgold)
		ld (reuben_gold),hl
		ret
	+:
	ex de,hl
	ld (reuben_gold),hl
	ret

Reuben_add_Bombs:
; takes: d
; destroys: a
	ld a,(reuben_bombs)
	add a,d
	ld d,a
	ld a,(reuben_maxbombs)
	
	jr c,{+}
	cp d
	jr c,{+}
	ld a,d
+:
	ld (reuben_bombs),a
	ret

Reuben_add_MP:
; takes: d
; destroys: a
	ld a,(reuben_mp)
	add a,d
	ld d,a
	ld a,(reuben_maxmp)
	
	jr c,{+}
	cp d
	jr c,{+}
	ld a,d
+:
	ld (reuben_mp),a
	ret

Reuben_pay_Gold:
; takes: hl
; destroys: hl, de
; preserves: a, bc, ix, iy
; returns: c is set if we can't pay, else c is reset
	ex de,hl
	ld hl,(reuben_gold)
	or a
	sbc hl,de
	ret c
	ld (reuben_gold),hl
	ret

Reuben_get_bottle:
; gets the current contents of the selected bottle (assumes one is selected)
; takes: none
; returns: a
; destroys: hl, de
; preserves: bc, ix, iy
	ld a,(reuben_curitem)
	ld d,0
	ld e,a
	ld hl,reuben_events - 1
	add hl,de
	ld a,(hl)
	and $0F
	ret

Reuben_set_bottle:
; overwrites the current contents of the selected bottle (assumes one is selected)
; takes: a
; destroys: hl, de, a
; preserves: bc, ix, iy
	push af
		ld a,(reuben_curitem)
		ld d,0
		ld e,a
		ld hl,reuben_events - 1
		add hl,de
		ld a,(hl)
		and $F0
		ld e,a
	pop af
	or e
	ld (hl),a
	ret

Reuben_upgrade_sword:
	ld a,(reuben_sword)
	cp 2
	jr nz,{+}
		; get axe
		ld a,4
		ld (reuben_sword),a
		doSpeech_real(string_find_axe)
		ret
	+:
	cp 4
	jr nz,{+}
		; get sword
		inc a
		ld (reuben_sword),a
		doSpeech_real(string_find_sword)
		ret
	+:
	cp 5
	jr nz,{+}
		; get breaded sword
		inc a
		ld (reuben_sword),a
		doSpeech_real(string_find_breadsword)
		ret
	+:
	cp 6
	jr nz,{+}
		; get toxic sword
		inc a
		ld (reuben_sword),a
		doSpeech_real(string_find_toxicsword)
		ret
	+:
	ret

Shop:
; takes:
; hl - ask dialog
; de - amount
; a - bottle?
	push af
	push de
		call _SpeechMove_entry
	pop hl
	ld a,e
	pop de
	or a
	ret z
	
	ld a,d
	or a
	jr z,{++}
		ld a,(reuben_curitem)
		or a
		jr z,{+}
		cp item_bottle4+1 ; +1 as we still want bottle4, else it wouldn't carry
		jr c,{++}
		
		+:
		doSpeech_real(string_need_bottle)
		xor a
		ret
	++:
	
	call Reuben_pay_Gold
	jr c,{+}
	doSpeech_real(string_shop_thanks)
	ld a,1
	or a
	ret
+:
	doSpeech_real(string_not_enough_gold)
	xor a ; we set z
	ret
	

.deflong shop_real(ask,price,bottle)
	ld hl,ask
	ld de,price
	.if bottle
		ld a,1
	.else
		xor a
	.endif
	call Shop
	ret z
.enddeflong

Increase_Speed_Arena:
	doSpeech_real(string_maru_intro)
	ld a,(reuben_wait)
	cp 201 ; we want to charge more if we have 200
	jr c,{+}
	doSpeech_real(string_maru_50gold)
	ld d,200
	ld hl,50
	jr {+1@}
+:
	cp 151 ; we want to charge more if we have 150
	jr c,{+}
	doSpeech_real(string_maru_100gold)
	ld d,150
	ld hl,100
	jr {+1@}
+:
	cp 101 ; we want to charge more if we have 100
	jr c,{+}
	doSpeech_real(string_maru_300gold)
	ld d,100
	ld hl,300
	jr {+1@}
+:
	cp 51 ; we want to charge more if we have 50
	jr c,{+}
	doSpeech_real(string_maru_500gold)
	ld d,50
	ld hl,500
	jr {+1@}
	
+:
	doSpeech_real(string_maru_noteach)
	ret
	; doSpeech_real(string_maru_more gold)
@:
	ld a,e
	or a
	ret z
	push de
		call Reuben_pay_Gold
		jr c,MaruNopay
		doSpeech_real(string_maru_thanks_buying)
		ld a,e
		or a
		jr z,{+}
	-:	doSpeech_real(string_maru_rules)
		ld a,e
		or a
		jr nz,{-}
	+:
		ld hl,(reuben_hp)
		ld (tmp6),hl
		ld hl,(reuben_maxhp)
		ld (reuben_hp),hl
	pop af
	push af
		; we now have in a what our next wait is
		ld hl,battle_flags
		set 0,(hl)
		ld hl,0
		
		cp 200
		jr z,{+}
		
		inc hl
		cp 150
		jr z,{+}
		
		inc hl
		cp 100
		jr z,{+}
		
		inc hl
	+:
		add hl,hl ; times 2
		ld de,MaruEnemyLut
		add hl,de
		ld e,(hl)
		inc hl
		ld d,(hl)
		ex de,hl
		ld a,(hl)
		-:
			inc hl
			push hl
				push af
					doSpeech_real(string_maru_enemy)
				pop af
				push bc
					bcall(_BattleEngine)
					push af
						bcall(_ClrBackBuffers)
						bcall(_DrawTileMap)
						call Update
					pop af
				pop bc
				cp 1
				jr z,MaruDie
			pop hl
			ld a,(hl)
			cp $ff
			jr nz,{-}
			
	pop af
	ld (reuben_wait),a
	ld hl,(tmp6)
	ld (reuben_hp),hl
	ld hl,battle_flags
	res 0,(hl)
	call _Move_Draw_Reuben
	doSpeech_real(string_maru_success)
	ret
MaruNopay:
	pop hl
	doSpeech_real(string_not_enough_gold)
	ret
MaruDie:
	ld hl,(tmp6)
	ld (reuben_hp),hl
	ld hl,battle_flags
	res 0,(hl)
	call _Move_Draw_Reuben
	doSpeech_real(string_maru_dead)
	jp _Move_Loop_fixstack ; i don't care where the stack pointer actually was
MaruEnemies1:
	.db enemy_Mouse
	.db enemy_Gecko
	.db enemy_Frog
	.db enemy_Small_Spiders
	.db enemy_mountain_bee
	.db enemy_mountain_snake
	.db enemy_mountain_bee
	.db $ff
MaruEnemies2:
	.db enemy_weird_caterpillar_thing
	.db enemy_air_snake_thing
	.db enemy_slime_blob
	.db enemy_eyeball_octopus
	.db $ff
MaruEnemies3:
	.db enemy_eyeballs_1_2
	.db enemy_eyeballs_2_2
	.db enemy_past_castle_1
	.db enemy_past_castle_2
	.db $ff
MaruEnemies4:
	.db enemy_cactus
	.db enemy_camel
	.db enemy_desertbug
	.db enemy_birdeye_thing_2
	.db $ff
MaruEnemyLut:
	.dw MaruEnemies1
	.dw MaruEnemies2
	.dw MaruEnemies3
	.dw MaruEnemies4



First_battle_aerilon_king:
	doSpeech_real(string_aerilon_castle_king_2)
	
	doBattle_real(enemy_aerilon_king_1)
	
	doSpeech_real(string_aerilon_castle_king_3)
	
	; we are already on world_aerilon_castle
	;ld a,world_aerilon_castle
	;ld (reuben_world),a
	
	ld a,(GSContrast)
	or %11000000
	-:
		push af
			call Sleep_short
		pop af
		inc a
		out ($10),a
		cp $ff
		jr nz,{-}
	ld a,tilemap_264
	ld (TileMapData),a
	bcall(_LoadTileMap)
	bcall(_DrawTileMap)
	call Update
	ld a,%00111111
	ld hl,GSContrast
	-:
		push af
			call Sleep_short
		pop af
		dec a
		or %11000000
		out ($10),a
		and %00111111
		cp (hl)
		jr nz,{-}
	
	ld bc,getCoordinates(5*8,6*8)
	ld d,16
	-:
		push de
			dec c
			push bc
				ld hl,sprites + (16*sprite_king)
				bcall(_graySpriteOverwrite_frontBuffer)
				call Sleep_short
				call Update
			pop bc
		pop de
		dec d
		jr nz,{-}
	ld b,8
	-:
		push bc
			ld bc,getCoordinates(5*8,4*8)
			ld hl,sprites + (16*sprite_king)
			bcall(_graySpriteOverwrite_frontBuffer)
			call Sleep_short
			call Update
			call Sleep_short
		pop bc
		djnz {-}
	
	ld bc,getCoordinates(5*8,6*8)
	doSpeech_real(string_aerilon_castle_king_4)
	
	ld bc,getCoordinates(5*8,3*8)
	push bc
	pop ix
	ld h,b
	ld l,c
	ld d,b
	ld e,c
	ld a,24
	-:
		push af
			dec b
			dec c
			push bc
			push hl
			push de
			push ix
				ld hl,sprites + (16*sprite_167)
				bcall(_graySpriteOverwrite_frontBuffer)
			pop ix
			pop de
			pop hl
			
			dec h
			inc l
			push hl
			push de
			push ix
				ld b,h
				ld c,l
				ld hl,sprites + (16*sprite_167)
				bcall(_graySpriteOverwrite_frontBuffer)
			pop ix
			pop de
			
			inc d
			dec e
			push de
			push ix
				ld b,d
				ld c,e
				ld hl,sprites + (16*sprite_167)
				bcall(_graySpriteOverwrite_frontBuffer)
			
			pop ix
			ld de,%0000000100000001
			add ix,de
			push ix
				push ix
				pop bc
				ld hl,sprites + (16*sprite_167)
				bcall(_graySpriteOverwrite_frontBuffer)
				call Sleep_short
				call Update
				
			pop ix
			
			pop de
			pop hl
			pop bc
		pop af
		dec a
		jr nz,{-}
	
	
	ld bc,getCoordinates(5*8,6*8)
	
	setEvent_real(event_portal_destroyed_offset,event_portal_destroyed_bit)
	setEvent_real(event_aerilon_west_open_offset,event_aerilon_west_open_bit)
	clearEvent_real(event_aerilon_east_closed_offset,event_aerilon_east_closed_bit)
	jp _Move_ReloadMap

Portal_Fix:
	doSpeech_real(string_portal_fix)
	push bc
		ld bc,getCoordinates(5*8 - 24,3*8 - 24)
		ld ix,getCoordinates(5*8 + 24,3*8 + 24)
		ld hl,getCoordinates(5*8 - 24,3*8 + 24)
		ld de,getCoordinates(5*8 + 24,3*8 - 24)
		
		ld a,24
		-:
			push af
				inc b
				inc c
				push bc
				push hl
				push de
				push ix
					ld hl,sprites + (16*sprite_167)
					bcall(_graySpriteOverwrite_frontBuffer)
				pop ix
				pop de
				pop hl
				
				inc h
				dec l
				push hl
				push de
				push ix
					ld b,h
					ld c,l
					ld hl,sprites + (16*sprite_167)
					bcall(_graySpriteOverwrite_frontBuffer)
				pop ix
				pop de
				
				dec d
				inc e
				push de
				push ix
					ld b,d
					ld c,e
					ld hl,sprites + (16*sprite_167)
					bcall(_graySpriteOverwrite_frontBuffer)
				
				pop ix
				
				push ix
				pop bc
				dec b
				dec c
				push bc
					
					ld hl,sprites + (16*sprite_167)
					bcall(_graySpriteOverwrite_frontBuffer)
					call Sleep_short
					call Update
					
				pop ix
				
				pop de
				pop hl
				pop bc
			pop af
			dec a
			jr nz,{-}
		
		call RevUpdate
		ld b,5
		-:
			push bc
				bcall(_ClrFrontBuffers)
				call Sleep_short
				call Update
				call Sleep_short
			pop bc
			djnz {-}
	pop bc
	clearEvent_real(event_portal_destroyed_offset,event_portal_destroyed_bit)
	jp _Move_ReloadMap

Battle_Instructions:
	doSpeech_real(string_person_dogglen_battle_instructions_ask)
	xor a
	cp e
	ret nz
	ld (battleReubenState),a ; a is already zero here now anyways
	ld (battleReubCurWait),a
	ld (battlePoison),a
	ld (battleStunCounter),a
	ld (battleEnemyStun),a
	bcall(_fade_to_white)
	push bc
		bcall(_Battle_Instructions_load_Mouse)
		bcall(_fade_from_white)
		
		call Sleep_med
		ld hl,string_person_dogglen_battle_instructions_bar
		ld a,%00000010
		call Speech
		bcall(__Battle_Redraw_frontbuffer)
		ld bc,getCoordinates(61,26)
		-:
			call Sleep_short
			push bc
				ld e,1
				ld ix,frontBuffer1
				call drawPxlOR
			pop bc
			inc b
			ld a,95
			cp b
			jr nz,{-}
		
		ld a,1
		ld (battleReubenState),a
		bcall(__Battle_Redraw_frontbuffer)
		call Sleep_med
		ld hl,string_person_dogglen_battle_instructions_attack_menu
		ld a,%00000010
		call Speech
		bcall(__Battle_Redraw_frontbuffer)
		call Sleep_med
		bcall(__Battle_Player_Attack_animation)
		ld hl,42
		ld bc,getCoordinates(52,16)
		xor a
		bcall(_NumText)
		call Sleep_long
		
		bcall(_fade_to_white)
		bcall(_ClrBackBuffers)
		bcall(_DrawTileMap)
		call Update
	pop bc
	call _Move_Draw_Reuben
	bcall(_fade_from_white)
	doSpeech_real(string_person_dogglen_battle_instructions_done)
	ret

Tradequest_item:
; INPUT:
; a - number
; hl - without item
; iy - with item ask
; ix - thank for item give
; de - thank for item later
; DESTROYS:
; all
	push bc
		ld b,a
		ld a,(reuben_tradequest)
		cp b
		jr nc,{+} ; we already previously gave that person that tradequest item
		ld d,b
	pop bc
	push de
	push af ; one byte smaller to save than th re-fetch
	push iy
	push ix
		call _SpeechMove_entry
	pop ix
	pop hl
	pop af
	pop de
	dec d
	cp d
	ret nz ; we don't have the item yet anyways
	; ok, now we have the item, so time to do stuff!
	push ix
		call _SpeechMove_entry
		ld a,e
	pop de
	or a
	ret nz ; we don't want to give it! How evil!
	ld hl,reuben_tradequest
	inc (hl)
	ex de,hl
	jp _SpeechMove_entry
	
	+:
	pop bc
	ex de,hl
	jp _SpeechMove_entry


Searchquest_person:
	isEvent_real(event_search_quest_offset,event_search_quest_bit)
	jr nz,{+}
		doSpeech_real(string_person_aerilon_moveless)
		ld a,e
		or a
		ret nz
		setEvent_real(event_search_quest_offset,event_search_quest_bit)
		ret
	+:
	ld a,(reuben_fright)
	cp 1
	jr nc,_Searchquest_upgrade1
_Searchquest_upgrade1_cont:
	cp 3
	jr nc,_Searchquest_upgrade2
_Searchquest_upgrade2_cont:
	cp 5
	jr nc,_Searchquest_upgrade3
_Searchquest_upgrade3_cont:
	cp 7
	jr nc,_Searchquest_upgrade4
_Searchquest_upgrade4_cont:
	cp 11
	jr nc,_Searchquest_upgrade5
	
	doSpeech_real(string_person_aerilon_moveless_thanks)
	ret
_Searchquest_upgrade1:
	ld hl,(reuben_maxgold)
	ld de,200
	sbc hl,de ; carry is re-set anyways
	jr nc,_Searchquest_upgrade1_cont ; already have the bigger wallet
	doSpeech_real(string_person_aerilon_moveless_wallet_200)
	ld hl,200
	ld (reuben_maxgold),hl
	ret
_Searchquest_upgrade2:
	ld hl,(reuben_maxgold)
	ld de,500
	sbc hl,de ; carry is re-set anyways
	jr nc,_Searchquest_upgrade2_cont ; already have the bigger wallet
	doSpeech_real(string_person_aerilon_moveless_wallet_500)
	ld hl,500
	ld (reuben_maxgold),hl
	ret
_Searchquest_upgrade3:
	ld hl,(reuben_maxgold)
	ld de,999
	sbc hl,de ; carry is re-set anyways
	jr nc,_Searchquest_upgrade3_cont
	doSpeech_real(string_person_aerilon_moveless_wallet_999)
	ld hl,999
	ld (reuben_maxgold),hl
	ret
_Searchquest_upgrade4:
	ld d,a
	ld a,(reuben_maxbombs)
	cp 30
	ld a,d
	jr nc,_Searchquest_upgrade4_cont ; already have the bigger bomb bag
	doSpeech_real(string_person_aerilon_moveless_bombs_30)
	ld a,30
	ld (reuben_maxbombs),a
	ret
_Searchquest_upgrade5:
	ld a,(reuben_maxbombs)
	cp 99
	jr nc,_Searchquest_done
	doSpeech_real(string_person_aerilon_moveless_bombs_99)
	ld a,99
	ld (reuben_maxbombs),a
	ret
_Searchquest_done:
	doSpeech_real(string_person_aerilon_moveless_done)
	ret


reuben_sandcastle_past_deactivate_forcefield:
	isEvent_real(event_sandcastle_past_forcefield_gen_1_offset,event_sandcastle_past_forcefield_gen_1_bit)
	ret z
	isEvent_real(event_sandcastle_past_forcefield_gen_2_offset,event_sandcastle_past_forcefield_gen_2_bit)
	ret z
	isEvent_real(event_sandcastle_past_forcefield_gen_3_offset,event_sandcastle_past_forcefield_gen_3_bit)
	ret z
	isEvent_real(event_sandcastle_past_forcefield_gen_1_offset,event_sandcastle_past_forcefield_gen_3_bit)
	ret z
	
	ld hl,script_sandcastle_past_forcefield
	bcall(_Script)
	setEvent_real(event_sandcastle_past_forcefield_offset,event_sandcastle_past_forcefield_bit)
	ret
